
;**************************************************************************
;*                    I2C Power Loss Indicator                            *
;**************************************************************************
;**************************************************************************
;*                                                                        *
;*          PLI                                                           *
;*          v 1.10 - 20.08.2013                                           *
;*                                                                        *
;*  based on:                                                             *
;*          I2C demo board                                                *
;*          v 2.17 - 08.08.2013                                           *
;*                                                                        *
;**************************************************************************
;*          Hardw.: PIC16F883 used                                        *
;*          OSC.......: Internal osc. used     POWER.....:   5V extern.   *
;**************************************************************************


;	SDA -- SDA (RC4)
;	SCL -- SCL (RC3)


 INCLUDE "P16F883.INC"


_U001			     EQU     B'1110000011000100'	;see Page 206 of Datasheet
_U002			     EQU     B'1111111111111111'	;see Page 207 of Datasheet

;	__CONFIG	_CONFIG1, _DEBUG_OFF & _PLL_EN & _BORV_2_5 & _BOR_OFF & _CP_OFF & _MCLR_DIS & _PWRT_DIS & _WDT_OFF & _INTRC_OSC_NOCLKOUT
;	__CONFIG	_CONFIG2, _VCAP_DIS


	__CONFIG	_CONFIG1, _U001

	__CONFIG	_CONFIG2, _U002


;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"

;interrupt
W_TEMP		EQU	H'22'
STATUS_TEMP	EQU	H'23'
PCLATH_TEMP	EQU	H'24'
data_tmp	EQU	H'25'
bus_D		EQU	H'26'
cnt1		EQU	H'27'
cnt2		EQU	H'28'
cnt3		EQU	H'29'
kbd_cnt1	EQU	H'2A'
kbd_buffer	EQU	H'2B'		;keyboard byte buffer
kbd_bb		EQU	H'2c'		;keyboard bit buffer
disp_pos	EQU	H'2D'
c_tmp		EQU	H'2E'
;---------------------------------------------------------------------
;---------------------------------------------------------------------
;RTC TIME STAMP register
v_ts_min	EQU	H'2F'
v_ts_hrs	EQU	H'30'
v_ts_day	EQU	H'31'
v_ts_mnt	EQU	H'32'
;
v_ee_buf	EQU	H'33'
;
v_dnum		EQU	H'34'

v_rtcrc		EQU	H'35'


;____ F R E E _____

;---------------------------------------------------------------------
;---------------------------------------------------------------------
v_hom_bcnt	EQU	H'42'		;blink counter
v_hom_blink	EQU	H'43'		;blink
v_hom_PIR1	EQU	H'44'
v_i2c_add	EQU	H'45'		;I2C address

;____ F R E E _____

;
pt_mode		EQU	H'48'		;bit0 = 1: M4 / bit1 = 1: L2
;
kaf		EQU	H'49'		;keyboard activity flag
;
vs_d0		EQU	H'4A'
vs_d1		EQU	H'4B'
vs_d2		EQU	H'4C'
vs_d3		EQU	H'4D'
;
kbd_out		EQU	H'4E'
kbd_parity	EQU	H'4F'

b_ax		EQU	H'50'
tmp_ot		EQU	H'51'

;-------------------------------------------------------------------
;-------------------------------------------------------------------
v_ram_add	EQU	H'52'		;RAM address in RTC
v_ram_dat	EQU	H'53'		;RAM data from/to RTC

v_pl_cnt	EQU	H'54'		;number of power loss events recorded
v_sd_day	EQU	H'55'		;start date: day
v_sd_mon	EQU	H'56'		;start date: month
;
v_sdd1		EQU	H'57'
v_sdd2		EQU	H'58'
v_smm1		EQU	H'59'
v_smm2		EQU	H'5A'
v_smm3		EQU	H'5B'
;-------------------------------------------------------------------
;-------------------------------------------------------------------
v_wd1		EQU	H'5C'
v_wd2		EQU	H'5D'
;-------------------------------------------------------------------
v_ee_add	EQU	H'5E'		;EEPROM pointer
;-------------------------------------------------------------------

v_f_init	EQU	H'5F'

;RTC variables --------------------
v_rtc_d1	EQU	H'60'
v_rtc_d2	EQU	H'61'
v_rtc_m1	EQU	H'62'
v_rtc_m2	EQU	H'63'
v_rtc_y1	EQU	H'64'
v_rtc_y2	EQU	H'65'
v_rtc_y3	EQU	H'66'
v_rtc_y4	EQU	H'67'
v_rtc_h1	EQU	H'68'
v_rtc_h2	EQU	H'69'
v_rtc_n1	EQU	H'6A'
v_rtc_n2	EQU	H'6B'
v_rtc_s1	EQU	H'6C'
v_rtc_s2	EQU	H'6D'

v_rtc_sec	EQU	H'6E'
v_rtc_min	EQU	H'6F'
v_rtc_hrs	EQU	H'70'
v_rtc_wday	EQU	H'71'	;day in the week
v_rtc_day	EQU	H'72'
v_rtc_mnt	EQU	H'73'
v_rtc_year	EQU	H'74'

v_rtc_wd	EQU	H'75'
;---------------------------------
v_tmp1		EQU	H'76'	;temp variable - general usage

v_rtc_t1	EQU	H'77'
v_rtc_t2	EQU	H'78'

v_k1		EQU	H'79'
v_k2		EQU	H'7A'
v_k3		EQU	H'7B'
v_k4		EQU	H'7C'
v_k5		EQU	H'7D'

v_qq1		EQU	H'7E'
v_qq2		EQU	H'7F'
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------


;==========================================================================

		ORG	0		;Reset vector address
		GOTO	RESET		;goto RESET routine when boot.

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;----------------------------------------------------------------------------
; INTERRUPT SERVICE ROUTINE -------------------------------------------------
;----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

		ORG	004

		retfie


;save important variables
		MOVWF	W_TEMP			;Copy W to W_TEMP register
		SWAPF	STATUS,W		;Swap status to be saved into W

;Swaps are used because they do not affect the status bits

		BANKSEL	STATUS_TEMP		;Select regardless of current bank
		MOVWF	STATUS_TEMP		;Copy status to bank zero STATUS_TEMP register
		MOVF	PCLATH,W		;Copy PCLATH to W register
		MOVWF	PCLATH_TEMP		;Copy W register to PCLATH_TEMP

		bcf	INTCON,GIE

		PAGESEL	PAGE0

		goto	int_serv
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************

RESET		nop
PAGE0
;
		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'01011111'	;Set PIC options (See datasheet).
		MOVWF	OPTION_REG	;Write the OPTION register.
		bcf	OPTION_REG,T0CS	;set-up TIMER0 as timer
		bsf	OPTION_REG,PSA	;prescaler NOT for TIMER0
		;
		CLRF	INTCON		;Disable interrupts
		clrf	IOCB
		bsf	IOCB,D'000'	;enable interrupt on change for RB0 only

		MOVLW	B'11000000'	;PORTA
		MOVWF	TRISA

		MOVLW	B'11111111'	;RB0 = keyboard clock; RB1 = kbd data
		MOVWF	TRISB

		MOVLW	B'00011000'	;PORTC
		MOVWF	TRISC


		MOVLW	B'11111000'	;PORTE = output; RE3 = input
		MOVWF	TRISE

		movlw	B'00000111'
		movwf	WPUB

		bsf	STATUS,RP0
		bsf	STATUS,RP1
		clrf	ANSEL
		clrf	ANSELH

		bcf	STATUS,RP0	;Switch Back to reg. Bank 0
		bcf	STATUS,RP1

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;set up I2C BUS for communication as a master
		movlw	B'00101000'		;see Datasheet Page 177
;			bit 3-0 SSPM<3:0>: Synchronous Serial Port Mode Select bits
;			1000 = I2C Master mode, clock = FOSC / (4 * (SSPADD+1))
		movwf	SSPCON

;set-up I2C speed
		bsf	STATUS,RP0
		movlw	B'00111111'
		movwf	SSPADD
		clrf	SSPSTAT
		bcf	STATUS,RP0
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------


; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; XXXXXXXXXX       initialization  XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

init_main	clrf	PORTC
		clrf	PORTA
		clrf	PORTB
;
		clrf	kaf

		clrf	PIE1
		clrf	PIE2

		bcf	INTCON,GIE		;disable all interrupts
		bsf	INTCON,PEIE

		bsf	STATUS,RP0
		clrf	PIE1
		clrf	PIE2
		bsf	PIE2,EEIE
		bcf	STATUS,RP0
;

;-----------------------------------------------------------------------------
;init display - SCREEN1
;
		PAGESEL	PAGE1
		call	disp_init
		call	w_release
		PAGESEL	PAGE0
;-------------------------------------------------------------
;-------------------------------------------------------------
		movlw	B'11011110'		;RTC = 1101111x
		movwf	v_i2c_add

		call	rtc_v_init
		call	power_up_init
;-------------------------------------------------------------------------------
;--- init phase is finished now ------------------------------------------------
;-------------------------------------------------------------------------------
		call	dr2
		call	dr2

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                          Main Screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
m001s		nop

		PAGESEL	PAGE1
		call	clear_disp
		PAGESEL	PAGE0

		movlw	D'001'
		movwf	v_rtcrc			;make sure RTC will be readed first time...

m001		nop
		movf	v_pl_cnt,0
		btfss	STATUS,Z
		goto	m001_nz
		nop
		PAGESEL	PAGE1
		call	w_pl_zero
		call	w_date_time
		PAGESEL	PAGE0
		goto	m001_7

m001_nz		nop
		PAGESEL	PAGE1
		call	w_pl_nz
		call	w_date_time
		PAGESEL	PAGE0

m001_7		nop
		decfsz	v_rtcrc,1
		goto	m001_8			;we do not need to read the RTC device all the time...

		call	rtc_i2c_read
		call	rtc_translate		;translate RTC date from RTC device to display format
		call	rtc_translate2

		movlw	D'003'
		movwf	v_rtcrc

m001_8		btfss	PORTB,D'000'
		goto	d001e			;Event Details

		btfss	PORTB,D'002'
		goto	crs001			;Reset counter switch

		goto	m001

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                    Counter Reset Screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
crs001		nop
		PAGESEL	PAGE1
		call	clear_disp
		call	w_crs
		PAGESEL	PAGE0

crs002		btfsc	PORTB,D'002'
		goto	m001s			;reset switch has been released

		movf	PORTB,0
		andlw	B'00000011'

		btfss	STATUS,Z
		goto	crs002

		goto	r001e			;both buttons has been pushed
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                    Counter Reset
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;button 2 has been pressed

r001e		call	first_init
		call	cnt_clr_init

;clean up the EEPROM data
		movlw	H'FF'
		movwf	v_ee_buf
		clrf	v_ee_add
;
r002		call	write_eeprom
		incf	v_ee_add,1
		btfss	STATUS,Z
		goto	r002

		PAGESEL	PAGE1
		call	clear_disp
		call	w_crse
		PAGESEL	PAGE0

crs005		btfss	PORTB,D'002'
		goto	crs005

		goto	m001s

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------
;                    Power Loss Event Detail Screen
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;button 1 has been pressed

d001e		movf	v_sd_day,0
		movwf	v_qq1
		movf	v_sd_mon,0
		movwf	v_qq2

		clrf	v_dnum

		nop
		PAGESEL	PAGE1
		call	clear_disp
		PAGESEL	PAGE0

		clrf	v_ee_add
;----------------------------------------------------
d001		incf	v_dnum,1

		movf	v_dnum,0
		xorlw	D'033'
		btfsc	STATUS,Z
		clrf	v_dnum

		call	read_ts_ee

		movf	v_ts_day,0
		movwf	v_sd_day
		movf	v_ts_mnt,0
		movwf	v_sd_mon
		call	rtc_translate3

		PAGESEL	PAGE1
		call	w_pl_detail1
		PAGESEL	PAGE0


		call	read_ts_ee

		movf	v_ts_day,0
		movwf	v_sd_day
		movf	v_ts_mnt,0
		movwf	v_sd_mon
		call	rtc_translate3

		PAGESEL	PAGE1
		call	w_pl_detail2
		PAGESEL	PAGE0


		call	dr2
		call	dr2


d005		btfss	PORTB,D'000'
		goto	d001
		btfss	PORTB,D'001'
		goto	d006
		goto	d005

d006		movf	v_qq1,0
		movwf	v_sd_day
		movf	v_qq2,0
		movwf	v_sd_mon
		goto	m001s
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
power_up_init	nop
		clrf	v_f_init

;1. - check control bytes 1 & 2 ----
		movlw	H'20'
		movwf	v_ram_add
		call	rtc_i2c_read_s
		movf	v_ram_dat,0
		xorlw	H'AA'
		btfss	STATUS,Z
		call	first_init		;first control byte missmatch
;
cnt_clr_init	movlw	H'21'
		movwf	v_ram_add
		call	rtc_i2c_read_s
		movf	v_ram_dat,0
		xorlw	H'75'
		btfss	STATUS,Z
		call	first_init		;second control byte missmatch
;get the number of power loss events --------
		movlw	H'22'
		movwf	v_ram_add
		call	rtc_i2c_read_s
		movf	v_ram_dat,0
		movwf	v_pl_cnt
;get the start date: day -------------------
		movlw	H'23'
		movwf	v_ram_add
		call	rtc_i2c_read_s
		movf	v_ram_dat,0
		movwf	v_sd_day
;get the start date: month -------------------
		movlw	H'24'
		movwf	v_ram_add
		call	rtc_i2c_read_s
		movf	v_ram_dat,0
		movwf	v_sd_mon
;get the EEPROM pointer
		movlw	H'25'
		movwf	v_ram_add
		call	rtc_i2c_read_s
		movf	v_ram_dat,0
		movwf	v_ee_add

		btfss	v_f_init,D'000'
		call	store_event

		return
;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
first_init	nop

		bsf	v_f_init,D'000'

		call	rtc_i2c_read

		movlw	H'20'
		movwf	v_ram_add
		movlw	H'AA'
		movwf	v_ram_dat
		call	rtc_i2c_write_s
;
		movlw	H'21'
		movwf	v_ram_add
		movlw	H'75'
		movwf	v_ram_dat
		call	rtc_i2c_write_s
;cnt
		movlw	H'22'
		movwf	v_ram_add
		movlw	H'00'
		movwf	v_ram_dat
		call	rtc_i2c_write_s
;set current date as start date
		movlw	H'23'
		movwf	v_ram_add
		movf	v_rtc_day,0
		movwf	v_ram_dat
		call	rtc_i2c_write_s
;
		movlw	H'24'
		movwf	v_ram_add
		movf	v_rtc_mnt,0
		movwf	v_ram_dat
		call	rtc_i2c_write_s
;pointer to the EEPROM
		movlw	H'25'
		movwf	v_ram_add
		movlw	H'00'
		movwf	v_ram_dat
		call	rtc_i2c_write_s

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
store_event	nop
;increment counter
		incf	v_pl_cnt,1
		movf	v_pl_cnt,0
		movwf	v_ram_dat
		movlw	H'22'
		movwf	v_ram_add
		call	rtc_i2c_write_s
;--------------------------------------------------
;get the events from RTC
		movlw	H'18'
		movwf	v_ram_add
		call	read_ts
;--------------------------------------------------
;write the first time stamp (power down) to EEPROM


		call	write_ts_ee

;--------------------------------------------------
;get the events from RTC
		movlw	H'1C'
		movwf	v_ram_add
		call	read_ts
;--------------------------------------------------
;write the second time stamp (power up) to EEPROM
		call	write_ts_ee

;write the EEPROM pointer back to RTC SRAM
		movlw	H'25'
		movwf	v_ram_add
		movf	v_ee_add,0
		movwf	v_ram_dat
		call	rtc_i2c_write_s

;-------------------------------------------------
;we have to cleat Vbat bit now to initialize time-stamp registers
;otherwise the registers will not be filled next time
		movlw	H'03'
		movwf	v_ram_add
		call	rtc_i2c_read_s
		bcf	v_ram_dat,D'004'
		call	rtc_i2c_write_s

		return

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;write time stamp to PIC EEPROM (4 bytes)
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
write_ts_ee	movf	v_ts_min,0
		movwf	v_ee_buf
		call	write_eeprom
		incf	v_ee_add,1
;
		movf	v_ts_hrs,0
		movwf	v_ee_buf
		call	write_eeprom
		incf	v_ee_add,1
;
		movf	v_ts_day,0
		movwf	v_ee_buf
		call	write_eeprom
		incf	v_ee_add,1
;
		movf	v_ts_mnt,0
		movwf	v_ee_buf
		call	write_eeprom
		incf	v_ee_add,1

		return

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;write a byte to the PIC EEPROM
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
write_eeprom	nop
;prepare data
		movf	v_ee_buf,0
		banksel	EEDAT
		movwf	EEDAT
		banksel	v_ee_add
		movf	v_ee_add,0
		banksel	EEADR
		movwf	EEADR

;enable EEPROM write
		BANKSEL	EECON1
		bcf	EECON1, EEPGD		;Point to DATA memory
		bsf	EECON1, WREN		;Enable writes
;EEPROM write
		MOVLW	H'55'
		MOVWF	EECON2
		MOVLW	H'AA'
		MOVWF	EECON2
		bsf	EECON1, WR		;Set WR bit to begin write

wee1		btfsc	EECON1,WR
		goto	wee1

		bcf	EECON1, WREN		;Disable writes
		bcf	STATUS, RP0		;Bank 0
		bcf	STATUS, RP1

		bcf	PIR2,EEIF

		return

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;read time stamp from PIC EEPROM (4 bytes)
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
read_ts_ee	call	read_eeprom
		movf	v_ee_buf,0
		movwf	v_ts_min
		incf	v_ee_add,1
;
		call	read_eeprom
		movf	v_ee_buf,0
		movwf	v_ts_hrs
		incf	v_ee_add,1
;
		call	read_eeprom
		movf	v_ee_buf,0
		movwf	v_ts_day
		incf	v_ee_add,1
;
		call	read_eeprom
		movf	v_ee_buf,0
		movwf	v_ts_mnt
		movlw	B'00011111'
		andwf	v_ts_mnt,1
		incf	v_ee_add,1

		return

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;read a byte from the PIC EEPROM
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
read_eeprom	nop
;prepare data
		banksel	v_ee_add
		movf	v_ee_add,0
		banksel	EEADR
		movwf	EEADR
;execute EEPROM read
		BANKSEL	EECON1
		bcf	EECON1, EEPGD		;Point to DATA memory
		bsf	EECON1, RD		;EE read

		BANKSEL	EEDAT
		movf	EEDAT, 0		;W = EEDAT

		bcf	STATUS, RP0		;Bank 0
		bcf	STATUS, RP1

		movwf	v_ee_buf

		return

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;	RTC routines
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

rtc_translate3	nop

;---------------------------------------
;hours
		swapf	v_ts_hrs,0
		andlw	B'00000011'
		addlw	'0'
		movwf	v_rtc_h1

		movf	v_ts_hrs,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_h2

;minutes
		swapf	v_ts_min,0
		andlw	B'00000111'
		addlw	'0'
		movwf	v_rtc_n1

		movf	v_ts_min,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_n2

;---------------------------------------

rtc_translate2	nop

;day
		swapf	v_sd_day,0
		andlw	B'00000011'
		addlw	'0'
		movwf	v_sdd1

		movf	v_sd_day,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_sdd2

;month
		movf	v_sd_mon,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	tt2jan
;
		movf	v_sd_mon,0
		xorlw	H'02'
		btfsc	STATUS,Z
		goto	tt2feb
;
		movf	v_sd_mon,0
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	tt2mar
;
		movf	v_sd_mon,0
		xorlw	H'04'
		btfsc	STATUS,Z
		goto	tt2apr
;
		movf	v_sd_mon,0
		xorlw	H'05'
		btfsc	STATUS,Z
		goto	tt2maj
;
		movf	v_sd_mon,0
		xorlw	H'06'
		btfsc	STATUS,Z
		goto	tt2jun
;
		movf	v_sd_mon,0
		xorlw	H'07'
		btfsc	STATUS,Z
		goto	tt2jul
;
		movf	v_sd_mon,0
		xorlw	H'08'
		btfsc	STATUS,Z
		goto	tt2aug
;
		movf	v_sd_mon,0
		xorlw	H'09'
		btfsc	STATUS,Z
		goto	tt2sep
;
		movf	v_sd_mon,0
		xorlw	H'10'
		btfsc	STATUS,Z
		goto	tt2okt
;
		movf	v_sd_mon,0
		xorlw	H'11'
		btfsc	STATUS,Z
		goto	tt2nov
;
		movf	v_sd_mon,0
		xorlw	H'12'
		btfsc	STATUS,Z
		goto	tt2dec
;
		movlw	'?'
		movwf	v_smm1
		movwf	v_smm2
		movwf	v_smm3
		return
;--------------------------------------------------------
tt2jan		movlw	'J'
		movwf	v_smm1
		movlw	'a'
		movwf	v_smm2
		movlw	'n'
		movwf	v_smm3
		return
;
tt2feb		movlw	'F'
		movwf	v_smm1
		movlw	'e'
		movwf	v_smm2
		movlw	'b'
		movwf	v_smm3
		return
;
tt2mar		movlw	'M'
		movwf	v_smm1
		movlw	'a'
		movwf	v_smm2
		movlw	'r'
		movwf	v_smm3
		return
;
tt2apr		movlw	'A'
		movwf	v_smm1
		movlw	'p'
		movwf	v_smm2
		movlw	'r'
		movwf	v_smm3
		return
;
tt2maj		movlw	'M'
		movwf	v_smm1
		movlw	'a'
		movwf	v_smm2
		movlw	'j'
		movwf	v_smm3
		return
;
tt2jun		movlw	'J'
		movwf	v_smm1
		movlw	'u'
		movwf	v_smm2
		movlw	'n'
		movwf	v_smm3
		return
;
tt2jul		movlw	'J'
		movwf	v_smm1
		movlw	'u'
		movwf	v_smm2
		movlw	'l'
		movwf	v_smm3
		return
;
tt2aug		movlw	'A'
		movwf	v_smm1
		movlw	'u'
		movwf	v_smm2
		movlw	'g'
		movwf	v_smm3
		return
;
tt2sep		movlw	'S'
		movwf	v_smm1
		movlw	'e'
		movwf	v_smm2
		movlw	'p'
		movwf	v_smm3
		return
;
tt2okt		movlw	'O'
		movwf	v_smm1
		movlw	'k'
		movwf	v_smm2
		movlw	't'
		movwf	v_smm3
		return
;
tt2nov		movlw	'N'
		movwf	v_smm1
		movlw	'o'
		movwf	v_smm2
		movlw	'v'
		movwf	v_smm3
		return
;
tt2dec		movlw	'D'
		movwf	v_smm1
		movlw	'e'
		movwf	v_smm2
		movlw	'c'
		movwf	v_smm3
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
rtc_translate	nop
		movf	v_rtc_sec,0
		xorlw	D'255'
		btfsc	STATUS,Z
		goto	rtc_v_init				;no RTC device connected

;day
		swapf	v_rtc_day,0
		andlw	B'00000011'
		addlw	'0'
		movwf	v_rtc_d1

		movf	v_rtc_day,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_d2


;month
		swapf	v_rtc_mnt,0
		andlw	B'00000001'
		addlw	'0'
		movwf	v_rtc_m1

		movf	v_rtc_mnt,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_m2

;year
		movlw	'2'
		movwf	v_rtc_y1
		movlw	'0'
		movwf	v_rtc_y2

		swapf	v_rtc_year,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_y3

		movf	v_rtc_year,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_y4

;hours
		swapf	v_rtc_hrs,0
		andlw	B'00000011'
		addlw	'0'
		movwf	v_rtc_h1

		movf	v_rtc_hrs,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_h2

;minutes
		swapf	v_rtc_min,0
		andlw	B'00000111'
		addlw	'0'
		movwf	v_rtc_n1

		movf	v_rtc_min,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_n2

;seconds
		swapf	v_rtc_sec,0
		andlw	B'00000111'
		addlw	'0'
		movwf	v_rtc_s1

		movf	v_rtc_sec,0
		andlw	B'00001111'
		addlw	'0'
		movwf	v_rtc_s2
;
;day in the week
		movf	v_rtc_wday,0
		andlw	B'00000111'
		movwf	v_rtc_wday

		movf	v_rtc_wday,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	rtc_set_Mo

		movf	v_rtc_wday,0
		xorlw	D'002'
		btfsc	STATUS,Z
		goto	rtc_set_Tu

		movf	v_rtc_wday,0
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	rtc_set_We

		movf	v_rtc_wday,0
		xorlw	D'004'
		btfsc	STATUS,Z
		goto	rtc_set_Th

		movf	v_rtc_wday,0
		xorlw	D'005'
		btfsc	STATUS,Z
		goto	rtc_set_Fr

		movf	v_rtc_wday,0
		xorlw	D'006'
		btfsc	STATUS,Z
		goto	rtc_set_Sa

		movf	v_rtc_wday,0
		xorlw	D'007'
		btfsc	STATUS,Z
		goto	rtc_set_Su
;
		return

rtc_set_Mo	movlw	'M'
		movwf	v_rtc_t1
		movlw	'o'
		movwf	v_rtc_t2
		return

rtc_set_Tu	movlw	'T'
		movwf	v_rtc_t1
		movlw	'u'
		movwf	v_rtc_t2
		return

rtc_set_We	movlw	'W'
		movwf	v_rtc_t1
		movlw	'e'
		movwf	v_rtc_t2
		return

rtc_set_Th	movlw	'T'
		movwf	v_rtc_t1
		movlw	'h'
		movwf	v_rtc_t2
		return

rtc_set_Fr	movlw	'F'
		movwf	v_rtc_t1
		movlw	'r'
		movwf	v_rtc_t2
		return

rtc_set_Sa	movlw	'S'
		movwf	v_rtc_t1
		movlw	'a'
		movwf	v_rtc_t2
		return

rtc_set_Su	movlw	'S'
		movwf	v_rtc_t1
		movlw	'u'
		movwf	v_rtc_t2
		return

;-----------------------------------------------------------------------------
rtc_backtrs	nop
;date - day
		movlw	'0'
		subwf	v_rtc_d1,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_d2,0
		addwf	v_tmp1,0

		movwf	v_rtc_day
;----------------------------------
;date - month
		movlw	'0'
		subwf	v_rtc_m1,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_m2,0
		addwf	v_tmp1,0

		movwf	v_rtc_mnt
;----------------------------------
;date - year
		movlw	'0'
		subwf	v_rtc_y3,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_y4,0
		addwf	v_tmp1,0

		movwf	v_rtc_year
;----------------------------------
;date - day in the week - always use 1; and we will see...
		movlw	'0'
		subwf	v_rtc_wd,0
		movwf	v_rtc_wday

;----------------------------------
;time - hours
		movlw	'0'
		subwf	v_rtc_h1,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_h2,0
		addwf	v_tmp1,0

		movwf	v_rtc_hrs
;----------------------------------
;time - minutes
		movlw	'0'
		subwf	v_rtc_n1,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_n2,0
		addwf	v_tmp1,0

		movwf	v_rtc_min
;----------------------------------
;time - seconds
		movlw	'0'
		subwf	v_rtc_s1,0
		movwf	v_tmp1
		swapf	v_tmp1,1

		movlw	'0'
		subwf	v_rtc_s2,0
		addwf	v_tmp1,0

		movwf	v_rtc_sec

		return
;-----------------------------------------------------------------------------
rtc_v_init	movlw	'-'
		movwf	v_rtc_d1
		movwf	v_rtc_d2
		movwf	v_rtc_m1
		movwf	v_rtc_m2
		movwf	v_rtc_y1
		movwf	v_rtc_y2
		movwf	v_rtc_y3
		movwf	v_rtc_y4
		movwf	v_rtc_h1
		movwf	v_rtc_h2
		movwf	v_rtc_n1
		movwf	v_rtc_n2
		movwf	v_rtc_s1
		movwf	v_rtc_s2

		movwf	v_rtc_t1
		movwf	v_rtc_t2

		return

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; I2C communication routines
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;-----------------------------------------------------------------------------
i2c_start	bsf	STATUS,RP0
		bsf	SSPCON2, SEN
		bcf	STATUS,RP0
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_stop	bsf	STATUS,RP0
		bsf	SSPCON2, PEN
		bcf	STATUS,RP0
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_ack		bsf	STATUS,RP0
		bcf	SSPCON2, ACKDT		;ACK
		bsf	SSPCON2, ACKEN
		bcf	STATUS,RP0
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_not_ack	bsf	STATUS,RP0
		bsf	SSPCON2, ACKDT		;NOT ACK
		bsf	SSPCON2, ACKEN
		bcf	STATUS,RP0
		call	d55
		return
;-----------------------------------------------------------------------------
;output: received byte is in W register
;
i2c_receive	bsf	STATUS,RP0
		bsf	SSPCON2, RCEN
		bcf	STATUS,RP0
		call	d55
		movf	SSPBUF,0
		return
;-----------------------------------------------------------------------------
;input: transmitted byte have to be in W register
;
i2c_send	movwf	SSPBUF
		call	d55		;wait a while
		return
;-----------------------------------------------------------------------------
i2c_restart	bsf	STATUS,RP0
		bsf	SSPCON2, RSEN
		bcf	STATUS,RP0
		call	d55
		return
;-----------------------------------------------------------------------------





;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; Real Time clock I2C communication routines
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;read single byte from RTC RAM
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;import: v_ram_add -- RAM address in RTC
;export: v_ram_dat -- RAM data from RTC

rtc_i2c_read_s	nop

		call	i2c_start

		movf	v_i2c_add,0		;get the RTC device address
		andlw	B'11111110'		;set bit 0 to "0" -> write
		call	i2c_send

		movf	v_ram_add,0		;memory address (= v_ram_add)
		call	i2c_send

		call	i2c_stop
		call	i2c_start

		movf	v_i2c_add,0		;get the RTC device address
		iorlw	B'00000001'		;set bit 0 to "1" -> read
		call	i2c_send

		call	i2c_receive		;received byte
		movwf	v_ram_dat
		call	i2c_not_ack

		call	i2c_stop


		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;read time stamp bytes from RTC RAM
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;import: v_ram_add -- start address of times stamp (18H or 1CH)
;export: v_ts_min  -- time stamp data from RTC
;        v_ts_hrs  -- time stamp data from RTC
;        v_ts_day  -- time stamp data from RTC
;        v_ts_mnt  -- time stamp data from RTC

read_ts		nop

		call	i2c_start

		movf	v_i2c_add,0		;get the RTC device address
		andlw	B'11111110'		;set bit 0 to "0" -> write
		call	i2c_send

		movf	v_ram_add,0		;memory address (= v_ram_add)
		call	i2c_send

		call	i2c_stop
		call	i2c_start

		movf	v_i2c_add,0		;get the RTC device address
		iorlw	B'00000001'		;set bit 0 to "1" -> read
		call	i2c_send

		call	i2c_receive		;received byte
		movwf	v_ts_min
		call	i2c_ack

		call	i2c_receive		;received byte
		movwf	v_ts_hrs
		call	i2c_ack

		call	i2c_receive		;received byte
		movwf	v_ts_day
		call	i2c_ack

		call	i2c_receive		;received byte
		movwf	v_ts_mnt
		call	i2c_not_ack

		call	i2c_stop


		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;read current date/time
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
rtc_i2c_read	nop

;Step 1 - Start condition
		call	i2c_start

;Step 2: addressing the RTC device
		movf	v_i2c_add,0		;get the RTC device address
		andlw	B'11111110'		;set bit 0 to "0" -> write
		call	i2c_send

;Step 3: send memory location
		movlw	B'00000000'		;memory address (= 0)
		call	i2c_send

;Step 4: send STOP condition (stop the write)
		call	i2c_stop

;Step 5: START condition
		call	i2c_start

;Step 6: addressing the RTC device
		movf	v_i2c_add,0		;get the RTC device address
		iorlw	B'00000001'		;set bit 0 to "1" -> read
		call	i2c_send

;-------------------------------------
;Step 7: read data
;7.1 - sec
		call	i2c_receive		;received byte - sec
		movwf	v_rtc_sec
		call	i2c_ack

;7.2 - min
		call	i2c_receive		;received byte - min
		movwf	v_rtc_min
		call	i2c_ack

;7.3 - hours
		call	i2c_receive		;received byte
		movwf	v_rtc_hrs
		call	i2c_ack

;7.4 - day in a week
		call	i2c_receive		;received byte
		movwf	v_rtc_wday
		call	i2c_ack

;7.5 - day
		call	i2c_receive		;received byte
		movwf	v_rtc_day
		call	i2c_ack

;7.6 - month
		call	i2c_receive		;received byte
		movwf	v_rtc_mnt
		call	i2c_ack

;7.7 - year
		call	i2c_receive		;received byte
		movwf	v_rtc_year
		call	i2c_not_ack

;Step 8: send STOP condition (stop the read)
		call	i2c_stop

		return

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;write single byte to RTC RAM
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;import: v_ram_add -- RAM address in RTC
;        v_ram_dat -- RAM data to be written to RTC

rtc_i2c_write_s	nop
		call	i2c_start
		movf	v_i2c_add,0
		andlw	B'11111110'		;set bit 0 to "0" -> write
		call	i2c_send

		movf	v_ram_add,0		;memory address
		call	i2c_send

		movf	v_ram_dat,0
		call	i2c_send

		call	i2c_stop

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
rtc_i2c_write	nop

;Step 1: Start condition
		call	i2c_start
;
;Step 2: addressing the RTC device

		movf	v_i2c_add,0
		andlw	B'11111110'		;set bit 0 to "0" -> write
		call	i2c_send

;Step 3: send memory location
		movlw	B'00000000'		;memory address (= 0)
		call	i2c_send
;-------------------------------------
;Step 4: write data
;4.1 - sec
		movf	v_rtc_sec,0
;-----------------------------------------------------
		btfsc	v_i2c_add,D'001'
		addlw	B'10000000'		;for MCP7940 add 80H (to set the bit 7 to 1 --> enable oscilator)
;-----------------------------------------------------
		call	i2c_send

;4.2 - min
		movf	v_rtc_min,0
		call	i2c_send

;4.3 - hours
		movf	v_rtc_hrs,0
		call	i2c_send

;4.4 - day in a week
		movf	v_rtc_wday,0
;-----------------------------------------------------
		btfsc	v_i2c_add,D'001'
		addlw	B'00001000'		;for MCP7940 add 08H (to set the bit 3 to 1 --> enable battery backup)
;-----------------------------------------------------
		call	i2c_send

;4.5 - day
		movf	v_rtc_day,0
		call	i2c_send

;4.6 - month
		movf	v_rtc_mnt,0
		call	i2c_send

;4.7 - year
		movf	v_rtc_year,0
		call	i2c_send

;Step 5: send STOP condition (stop the read)
		call	i2c_stop

		return

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;		*********************************************
;		*               delay routine               *
;		*********************************************

;------------------------------------
d7		MOVLW   D'127'
		MOVWF   TIMER2
DEL_LOOP71	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP72	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP72
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP71
		return

;------------------------------------

d2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1
		return
;------------------------------------
dr2		call	d2
		call	d2
		call	d2
		return
;------------------------------------
dr2c		call	d2
		call	d2
		call	d2

		return
;------------------------------------

dr55		MOVLW   D'002'
		MOVWF   TIMER2
DEL_LOOPb1	MOVLW   D'100'
		MOVWF   TIMER1

DEL_LOOPb2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOPb2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOPb1
		return

;-------------------------------------------------------------------------------
d55		movlw	D'255'
		movwf	TIMER2
d55_loop	decfsz	TIMER2,F
		goto	d55_loop
		return
;
dr55x		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		return
;
dr2n		call	d2
		call	d2
		return
;------------------------------------
dr77		call	d2
		return
;------------------------------------
d55e		movlw	D'255'
		movwf	TIMER2
d55e_loop	decfsz	TIMER2,F
		goto	d55e_loop
		return
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------

int_serv	nop
		nop
		nop

		bcf	PIR1, SSPIF
		PAGESEL	PAGE1
		call	wm_I
		PAGESEL	PAGE0

;restore values
		BANKSEL STATUS_TEMP	;Select regardless of current bank
		MOVF PCLATH_TEMP,W	;
		MOVWF PCLATH		;Restore PCLATH
		SWAPF STATUS_TEMP,W	;Swap STATUS_TEMP register into W
					;(sets bank to original state)
		MOVWF STATUS		;Move W into STATUS register
		SWAPF W_TEMP,F		;Swap W_TEMP
		SWAPF W_TEMP,W		;Swap W_TEMP into W


		bsf	INTCON,GIE
		bsf	INTCON,PEIE

		bsf	PIE1,SSPIE
		retfie

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
d57		MOVLW   D'030'
		MOVWF   TIMER2
DEL_LOOP5721	MOVLW   D'255'
		MOVWF   TIMER1
DEL_LOOP5722	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP5722
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP5721
		return
;
dr01s		MOVLW   D'010'
		MOVWF   TIMER2
DEL_LOOP3_1s	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP3_2s	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP3_2s
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP3_1s
		RETURN
;
;----------------------------------------------------------------------------
dr01		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP3_1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP3_2	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP3_2
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP3_1
		RETURN
;-------------------------------------


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; display routines --- PAGE 1 --------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


ORG 800
PAGE1
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x                       LCD Display Communication Routines                                     x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;RS  = RE2 (C)
;R/W = RE1 (C)
;E   = RE0 (C)

;D0 - D3 -- not connected

;D4  = RD0 (A)
;D5  = RD1 (A)
;D6  = RD2 (A)
;D7  = RD3 (A)


disp_init	nop
;1st step of init:
;		wait for LCD hardware reset (after power on) - no software reset performed...
		clrf	PORTA
		clrf	PORTC
		call	d2_p2

;-- SW Reset to be able to enter 4-bit mode
		bcf	PORTC,D'001'
		bcf	PORTC,D'002'
		bcf	PORTC,D'000'	;EN = 0 => disable


		movlw	B'00000011'
		movwf	PORTA
		bsf	PORTC,D'000'
		call	dy_comm
		bcf	PORTC,D'000'
		call	d2_p2

		movlw	B'00000011'
		movwf	PORTA
		bsf	PORTC,D'000'
		call	dy_comm
		bcf	PORTC,D'000'
		call	d2_p2

		movlw	B'00000011'
		movwf	PORTA
		bsf	PORTC,D'000'
		call	dy_comm
		bcf	PORTC,D'000'
		call	d2_p2

		movlw	B'00000010'
		movwf	PORTA
		bsf	PORTC,D'000'
		call	dy_comm
		bcf	PORTC,D'000'
		call	d2_p2

;-----------------------------------------------------------------
;2nd step of init:
;     command:           '001DNFxx' (SET FUNCTION)
;                            D = 1	--> 8 bit interface
;                            D = 0	--> 4 bit interface
;			 '001DNFxx'
;                             N = 1	--> 2 line display
;                             N = 0	--> 1 line display
;			 '001DNFxx'
;                              F = 1	--> 5x10
;			       F = 0	--> 5x7
;			 '001DNFxx'
		movlw	B'00101000'		;2 lines - (for disp 1x16: 1-8  = 00h - 0Fh
;                                                                          9-16 = 40h - 4Fh )
		call	send_comm
;-----------------------------------------------------------------
;3rd step of init:
;     command:           '00001DCB' (DISPLAY ON/OFF)
;                              D = 1	--> Display on
;			       D = 0	--> Display off
;           		 '00001DCB'
;                               C = 1	--> Cursor on
;				C = 0	--> Cursor off
;           		 '00001DCB'
;				 B = 1	--> Blinking on
;				 B = 0	--> Blinking off
		movlw	B'00001000'
		call	send_comm

		movlw	B'00001111'
		call	send_comm
;-----------------------------------------------------------------
;4th step of init:
;     command:           '000001DS' (ENTRY MODE SET)
;				D = 1	--> Increase
;				D = 0	--> Decrease
;		         '000001DS'
;				 S = 1	--> display is shifted
;				 S = 0	--> display is not shifted
;		         '000001DS'
		movlw	B'00000110'
		call	send_comm
;-----------------------------------------------------------------
;clear the display
		movlw	B'00000001'
		call	send_comm
		return
;------------------------------------------------------------------------------------------------
;8-bit data transfer routines
;------------------------------------------------------------------------------------------------
send_data	bcf	PORTC,D'000'		;EN -> 0
		movwf	data_tmp

		swapf	data_tmp,0
		andlw	B'00001111'
		movwf	PORTA			;higher 4 bits are put to RA0-RA3
		call	scbd			;send

		movf	data_tmp,0
		andlw	B'00001111'
		movwf	PORTA			;lower 4 bits are put to RA0-RA3
		call	scbd			;send

		return

;set control bits
scbd		bcf	PORTC,D'001'		;R/W -> 0 (write)
		bsf	PORTC,D'002'		;RS -> 1 (data)
		bsf	PORTC,D'000'		;EN -> 1
		call	dy_comm			;wait a while...
		bcf	PORTC,D'000'		;EN -> 0
		call	d1			;wait a little bit...

		return
;------------------------------------------------------------------------------------------------
send_comm	bcf	PORTC,D'000'		;EN -> 0
		movwf	data_tmp

		swapf	data_tmp,0
		andlw	B'00001111'
		movwf	PORTA			;higher 4 bits are put to RA0-RA3
		call	scbc			;send

		movf	data_tmp,0
		andlw	B'00001111'
		movwf	PORTA			;lower 4 bits are put to RA0-RA3
		call	scbc			;send

		return

;set control bits
scbc		bcf	PORTC,D'001'		;R/W -> 0 (write)
		bcf	PORTC,D'002'		;RS -> 0 (command)
		bsf	PORTC,D'000'		;EN -> 1
		call	dy_comm			;wait a while...
		bcf	PORTC,D'000'		;EN -> 0
		call	d1			;wait a little bit...
		return
;-----------------------------------------------------------------------------------------------
;----------------------------------------------
dy_comm		call	dy_comm2	;01
		call	dy_comm2	;02
		call	dy_comm2	;03
		return

		call	dy_comm2	;04
		call	dy_comm2	;05
		return

		call	dy_comm2	;06
		call	dy_comm2	;07
		call	dy_comm2	;08
		call	dy_comm2	;09
		call	dy_comm2	;10
		call	dy_comm2	;11
		call	dy_comm2	;12
		return


dy_comm2	MOVLW   D'010'			;delay value changed: original value was: 255
		MOVWF   TIMER2
DEL_LOOP22c	DECFSZ  TIMER2,F
                GOTO    DEL_LOOP22c
		return
;---------------------------------------------------------------------------
;
d1		call	d1b		;01
		call	d1b		;02
		return

		call	d1b		;03
		return

		call	d1b		;04
		call	d1b		;05
		call	d1b		;06
		call	d1b		;07
		call	d1b		;08
		call	d1b		;09
		return


d1b		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP11	DECFSZ  TIMER2,F
                GOTO    DEL_LOOP11
		return
;
d2_p2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOPx1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOPx2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOPx2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOPx1
		return

;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
clear_disp	call	disp_clr1
		call	disp_clr2
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
n9_char		movlw	B'11000000'		;od znaku c.9 treba zmenit adresu na 40h
		call	send_comm

		movlw	B'10000000'
		movwf	disp_pos
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
init2		movlw	B'00000001'
;		call	send_comm
		movlw	B'10000000'
		call	send_comm
		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
;character set
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

wm_space	movlw	' '
		call	send_data
		return

wm_dvojbodka	movlw	':'
		call	send_data
		return

wm_point	nop
wm_bodka	movlw	'.'
		call	send_data
		return

wm_otaznik	movlw	'?'
		call	send_data
		return

wm_sls		movlw	'/'
		call	send_data
		return

wm_kk		movlw	'#'
		call	send_data
		return

wm_minus	movlw	'-'
		call	send_data
		return

wm_plus		movlw	'+'
		call	send_data
		return

wm_hvizdicka	movlw	'*'
		call	send_data
		return

wm_zat1		movlw	'('
		call	send_data
		return

wm_zat2		movlw	')'
		call	send_data
		return

wm_sip1		movlw	'>'
		call	send_data
		return

wm_sip2		movlw	'<'
		call	send_data
		return

wm_stup		movlw	H'DF'
		call	send_data
		return

wm_rs		movlw	'='
		call	send_data
		return

wm_and		movlw	'&'
		call	send_data
		return
;---------- cislice ----------------------------------------
wm_0		movlw	'0'
		call	send_data
		return

wm_1		movlw	'1'
		call	send_data
		return

wm_2		movlw	'2'
		call	send_data
		return

wm_3		movlw	'3'
		call	send_data
		return

wm_4		movlw	'4'
		call	send_data
		return

wm_5		movlw	'5'
		call	send_data
		return

wm_6		movlw	'6'
		call	send_data
		return

wm_7		movlw	'7'
		call	send_data
		return

wm_8		movlw	'8'
		call	send_data
		return

wm_9		movlw	'9'
		call	send_data
		return

;--------- velke pismena -----------------------------------
wm_A		movlw	'A'
		call	send_data
		return

wm_B		movlw	'B'
		call	send_data
		return

wm_C		movlw	'C'
		call	send_data
		return

wm_D		movlw	'D'
		call	send_data
		return

wm_E		movlw	'E'
		call	send_data
		return

wm_F		movlw	'F'
		call	send_data
		return

wm_G		movlw	'G'
		call	send_data
		return

wm_H		movlw	'H'
		call	send_data
		return

wm_I		movlw	'I'
		call	send_data
		return

wm_J		movlw	'J'
		call	send_data
		return

wm_K		movlw	'K'
		call	send_data
		return

wm_L		movlw	'L'
		call	send_data
		return

wm_M		movlw	'M'
		call	send_data
		return

wm_N		movlw	'N'
		call	send_data
		return

wm_O		movlw	'O'
		call	send_data
		return

wm_P		movlw	'P'
		call	send_data
		return

wm_Q		movlw	'Q'
		call	send_data
		return

wm_R		movlw	'R'
		call	send_data
		return

wm_S		movlw	'S'
		call	send_data
		return

wm_T		movlw	'T'
		call	send_data
		return

wm_U		movlw	'U'
		call	send_data
		return

wm_V		movlw	'V'
		call	send_data
		return

wm_W		movlw	'W'
		call	send_data
		return

wm_X		movlw	'X'
		call	send_data
		return

wm_Y		movlw	'Y'
		call	send_data
		return

wm_Z		movlw	'Z'
		call	send_data
		return

;--------- male pismena -----------------------------------

wm_a		movlw	'a'
		call	send_data
		return

wm_b		movlw	'b'
		call	send_data
		return

wm_c		movlw	'c'
		call	send_data
		return

wm_d		movlw	'd'
		call	send_data
		return

wm_e		movlw	'e'
		call	send_data
		return

wm_f		movlw	'f'
		call	send_data
		return

wm_g		movlw	'g'
		call	send_data
		return

wm_h		movlw	'h'
		call	send_data
		return

wm_i		movlw	'i'
		call	send_data
		return

wm_j		movlw	'j'
		call	send_data
		return

wm_k		movlw	'k'
		call	send_data
		return

wm_l		movlw	'l'
		call	send_data
		return

wm_m		movlw	'm'
		call	send_data
		return

wm_n		movlw	'n'
		call	send_data
		return

wm_o		movlw	'o'
		call	send_data
		return

wm_p		movlw	'p'
		call	send_data
		return

wm_r		movlw	'r'
		call	send_data
		return

wm_s		movlw	's'
		call	send_data
		return

wm_t		movlw	't'
		call	send_data
		return

wm_u		movlw	'u'
		call	send_data
		return

wm_v		movlw	'v'
		call	send_data
		return

wm_w		movlw	'w'
		call	send_data
		return

wm_x		movlw	'x'
		call	send_data
		return

wm_y		movlw	'y'
		call	send_data
		return

wm_z		movlw	'z'
		call	send_data
		return
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
convert2ascii	movwf	c_tmp
;
		xorlw	H'15'
		btfsc	STATUS,Z
		goto	wm_Q
;
		movf	c_tmp,0
		xorlw	H'1D'
		btfsc	STATUS,Z
		goto	wm_W
;
		movf	c_tmp,0
		xorlw	H'24'
		btfsc	STATUS,Z
		goto	wm_E
;
		movf	c_tmp,0
		xorlw	H'2D'
		btfsc	STATUS,Z
		goto	wm_R
;
		movf	c_tmp,0
		xorlw	H'2C'
		btfsc	STATUS,Z
		goto	wm_T
;
		movf	c_tmp,0
		xorlw	H'35'
		btfsc	STATUS,Z
		goto	wm_Y
;
		movf	c_tmp,0
		xorlw	H'3C'
		btfsc	STATUS,Z
		goto	wm_U
;
		movf	c_tmp,0
		xorlw	H'43'
		btfsc	STATUS,Z
		goto	wm_I
;
		movf	c_tmp,0
		xorlw	H'44'
		btfsc	STATUS,Z
		goto	wm_O
;
		movf	c_tmp,0
		xorlw	H'4D'
		btfsc	STATUS,Z
		goto	wm_P
;
		movf	c_tmp,0
		xorlw	H'16'
		btfsc	STATUS,Z
		goto	wm_1
;
		movf	c_tmp,0
		xorlw	H'1E'
		btfsc	STATUS,Z
		goto	wm_2
;
		movf	c_tmp,0
		xorlw	H'26'
		btfsc	STATUS,Z
		goto	wm_3
;
		movf	c_tmp,0
		xorlw	H'25'
		btfsc	STATUS,Z
		goto	wm_4
;
		movf	c_tmp,0
		xorlw	H'2E'
		btfsc	STATUS,Z
		goto	wm_5
;
		movf	c_tmp,0
		xorlw	H'36'
		btfsc	STATUS,Z
		goto	wm_6
;
		movf	c_tmp,0
		xorlw	H'3D'
		btfsc	STATUS,Z
		goto	wm_7
;
		movf	c_tmp,0
		xorlw	H'3E'
		btfsc	STATUS,Z
		goto	wm_8
;
		movf	c_tmp,0
		xorlw	H'46'
		btfsc	STATUS,Z
		goto	wm_9
;
		movf	c_tmp,0
		xorlw	H'45'
		btfsc	STATUS,Z
		goto	wm_0
;
		movf	c_tmp,0
		xorlw	H'1C'
		btfsc	STATUS,Z
		goto	wm_A
;
		movf	c_tmp,0
		xorlw	H'1B'
		btfsc	STATUS,Z
		goto	wm_S
;
		movf	c_tmp,0
		xorlw	H'23'
		btfsc	STATUS,Z
		goto	wm_D
;
		movf	c_tmp,0
		xorlw	H'2B'
		btfsc	STATUS,Z
		goto	wm_F
;
		movf	c_tmp,0
		xorlw	H'34'
		btfsc	STATUS,Z
		goto	wm_G
;
		movf	c_tmp,0
		xorlw	H'33'
		btfsc	STATUS,Z
		goto	wm_H
;
		movf	c_tmp,0
		xorlw	H'3B'
		btfsc	STATUS,Z
		goto	wm_J
;
		movf	c_tmp,0
		xorlw	H'42'
		btfsc	STATUS,Z
		goto	wm_K
;
		movf	c_tmp,0
		xorlw	H'4B'
		btfsc	STATUS,Z
		goto	wm_L
;
		movf	c_tmp,0
		xorlw	H'1A'
		btfsc	STATUS,Z
		goto	wm_Z
;
		movf	c_tmp,0
		xorlw	H'22'
		btfsc	STATUS,Z
		goto	wm_X
;
		movf	c_tmp,0
		xorlw	H'21'
		btfsc	STATUS,Z
		goto	wm_C
;
		movf	c_tmp,0
		xorlw	H'2A'
		btfsc	STATUS,Z
		goto	wm_V
;
		movf	c_tmp,0
		xorlw	H'32'
		btfsc	STATUS,Z
		goto	wm_B
;
		movf	c_tmp,0
		xorlw	H'31'
		btfsc	STATUS,Z
		goto	wm_N
;
		movf	c_tmp,0
		xorlw	H'3A'
		btfsc	STATUS,Z
		goto	wm_M
;
		movf	c_tmp,0
		xorlw	H'29'
		btfsc	STATUS,Z
		goto	wm_space
;
		movf	c_tmp,0
		xorlw	H'6C'
		btfsc	STATUS,Z
		goto	wm_7
;
		movf	c_tmp,0
		xorlw	H'75'
		btfsc	STATUS,Z
		goto	wm_8
;
		movf	c_tmp,0
		xorlw	H'7D'
		btfsc	STATUS,Z
		goto	wm_9
;
		movf	c_tmp,0
		xorlw	H'6B'
		btfsc	STATUS,Z
		goto	wm_4
;
		movf	c_tmp,0
		xorlw	H'73'
		btfsc	STATUS,Z
		goto	wm_5
;
		movf	c_tmp,0
		xorlw	H'74'
		btfsc	STATUS,Z
		goto	wm_6
;
		movf	c_tmp,0
		xorlw	H'69'
		btfsc	STATUS,Z
		goto	wm_1
;
		movf	c_tmp,0
		xorlw	H'72'
		btfsc	STATUS,Z
		goto	wm_2
;
		movf	c_tmp,0
		xorlw	H'7A'
		btfsc	STATUS,Z
		goto	wm_3
;
		movf	c_tmp,0
		xorlw	H'70'
		btfsc	STATUS,Z
		goto	wm_0
;
		movf	c_tmp,0
		xorlw	H'71'
		btfsc	STATUS,Z
		goto	wm_bodka
;
		movf	c_tmp,0
		xorlw	H'79'
		btfsc	STATUS,Z
		goto	wm_plus
;
		movf	c_tmp,0
		xorlw	H'7B'
		btfsc	STATUS,Z
		goto	wm_minus
;
		movf	c_tmp,0
		xorlw	H'7C'
		btfsc	STATUS,Z
		goto	wm_hvizdicka

		movf	c_tmp,0
		xorlw	H'76'			;ESC
		btfsc	STATUS,Z
;!!!!!!		goto	main_s


;not known character - either transmission error or not decoded... - do nothing...
		decf	disp_pos,1
		
		return


		return
;-----------------------------------------------------------------------------
;---------------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
w_i2c_add	call	wm_A
		call	wm_dvojbodka
		movf	v_i2c_add,0
		movwf	kbd_buffer

;-------------------------------------------------------------------------------
write_bin7	movlw	'0'
		btfsc	kbd_buffer,D'007'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'006'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'005'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'004'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'003'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'002'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'001'
		movlw	'1'
		call	send_data

		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_dec	nop

		movlw	'0'
		movwf	v_wd1
		movwf	v_wd2

		incf	kbd_buffer,1

wd_loop		decfsz	kbd_buffer,1
		goto	wd_cnt

		movf	v_wd1,0
		call	send_data
		movf	v_wd2,0
		call	send_data

		return
;-------------------------
wd_cnt		incf	v_wd2,1
		movf	v_wd2,0
		xorlw	':'
		btfss	STATUS,Z
		goto	wd_loop

		movlw	'0'
		movwf	v_wd2
		incf	v_wd1,1
		goto	wd_loop
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_hex	nop

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'00'
		btfsc	STATUS,Z
		call	wm_0

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'10'
		btfsc	STATUS,Z
		call	wm_1

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'20'
		btfsc	STATUS,Z
		call	wm_2

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'30'
		btfsc	STATUS,Z
		call	wm_3

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'40'
		btfsc	STATUS,Z
		call	wm_4

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'50'
		btfsc	STATUS,Z
		call	wm_5

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'60'
		btfsc	STATUS,Z
		call	wm_6

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'70'
		btfsc	STATUS,Z
		call	wm_7

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'80'
		btfsc	STATUS,Z
		call	wm_8

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'90'
		btfsc	STATUS,Z
		call	wm_9

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'A0'
		btfsc	STATUS,Z
		call	wm_A

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'B0'
		btfsc	STATUS,Z
		call	wm_B

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'C0'
		btfsc	STATUS,Z
		call	wm_C

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'D0'
		btfsc	STATUS,Z
		call	wm_D

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'E0'
		btfsc	STATUS,Z
		call	wm_E

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'F0'
		btfsc	STATUS,Z
		call	wm_F
;
		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'00'
		btfsc	STATUS,Z
		goto	wm_0

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	wm_1

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'02'
		btfsc	STATUS,Z
		goto	wm_2

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	wm_3

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'04'
		btfsc	STATUS,Z
		goto	wm_4

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'05'
		btfsc	STATUS,Z
		goto	wm_5

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'06'
		btfsc	STATUS,Z
		goto	wm_6

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'07'
		btfsc	STATUS,Z
		goto	wm_7

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'08'
		btfsc	STATUS,Z
		goto	wm_8

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'09'
		btfsc	STATUS,Z
		goto	wm_9

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0A'
		btfsc	STATUS,Z
		goto	wm_A

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0B'
		btfsc	STATUS,Z
		goto	wm_B

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0C'
		btfsc	STATUS,Z
		goto	wm_C

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0D'
		btfsc	STATUS,Z
		goto	wm_D

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0E'
		btfsc	STATUS,Z
		goto	wm_E

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0F'
		btfsc	STATUS,Z
		goto	wm_F


		return
;---------------------------------------------------------------------------------


;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; write texts to the display
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;---------------------------------------------------------------------------
w_pcsl		movlw	B'11001100'		;line 2 / char12
		call	send_comm

		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space

		return
;---------------------------------------------------------------------------
w_otaznik	call	send_comm
;
		call	wm_otaznik
;
		movf	tmp_ot,0
		call	send_comm
		return
;---------------------------------------------------------------------------
w_scr5		movlw	B'10001100'		;line 1 / char12
		call	send_comm


		return
;---------------------------------------------------------------------------
w_fvalid	btfss	pt_mode,D'001'
		goto	w_fvm4
;here it is for L2
		movlw	B'10001101'		;line 1 / char13
		call	send_comm

		call	wm_F
		call	wm_V
		call	wm_dvojbodka
		call	wm_D
		call	wm_0
		call	wm_minus
		call	wm_D
		call	wm_1
		call	wm_dvojbodka
		movf	vs_d0,0
		call	send_data
		movf	vs_d1,0
		call	send_data
		return
;
w_fvm4		movlw	B'10001011'		;line 1 / char11
		call	send_comm

		call	wm_F
		call	wm_V
		call	wm_dvojbodka
		call	wm_D
		call	wm_0
		call	wm_minus
		call	wm_D
		call	wm_3
		call	wm_dvojbodka
		movf	vs_d0,0
		call	send_data
		movf	vs_d1,0
		call	send_data
		movf	vs_d2,0
		call	send_data
		movf	vs_d3,0
		call	send_data
		return
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
w_kbd		movlw	B'11001100'		;line 1 / char12
		call	send_comm

		movlw	'0'
		btfsc	PORTB,D'000'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	PORTB,D'001'
		movlw	'1'
		call	send_data

		return
;--------------------------------------------------------------------------
w_data		movlw	B'11000000'		;line 2 / char0
		call	send_comm


		return
;--------------------------------------------------------------------------
w_search	movlw	B'10001100'		;line 1 / char12
		call	send_comm
		return
;
;------------------------------------------------------------------------------
;-----------------------------------------------------------------------------
w_search_again	movlw	B'11000000'		;line 2 / char0
		call	send_comm

		call	wm_R
		return
;---------------------------------------------------------------------------
disp_clr1	movlw	B'10000000'		;line 1 / char0
		call	send_comm

		call	wm_space	;01
		call	wm_space	;02
		call	wm_space	;03
		call	wm_space	;04
		call	wm_space	;05
		call	wm_space	;06
		call	wm_space	;07
		call	wm_space	;08
		call	wm_space	;09
		call	wm_space	;10
		call	wm_space	;11
		call	wm_space	;12
		call	wm_space	;13
		call	wm_space	;14
		call	wm_space	;15
		call	wm_space	;16
		call	wm_space	;17
		call	wm_space	;18
		call	wm_space	;19
		call	wm_space	;20
		call	wm_space	;21
		call	wm_space	;22
		call	wm_space	;23
		call	wm_space	;24

		return
;---------------------------------------------------------------------------
disp_clr2	movlw	B'11000000'		;line 2 / char0
		call	send_comm

		call	wm_space	;01
		call	wm_space	;02
		call	wm_space	;03
		call	wm_space	;04
		call	wm_space	;05
		call	wm_space	;06
		call	wm_space	;07
		call	wm_space	;08
		call	wm_space	;09
		call	wm_space	;10
		call	wm_space	;11
		call	wm_space	;12
		call	wm_space	;13
		call	wm_space	;14
		call	wm_space	;15
		call	wm_space	;16
		call	wm_space	;17
		call	wm_space	;18
		call	wm_space	;19
		call	wm_space	;20
		call	wm_space	;21
		call	wm_space	;22
		call	wm_space	;23
		call	wm_space	;24

		return

;-------------------------------------------------------------------------------
w_release	call	init2

		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_I
		call	wm_n
		call	wm_d
		call	wm_i
		call	wm_k
		call	wm_a
		call	wm_t
		call	wm_o
		call	wm_r
		call	wm_space
		call	wm_v
		call	wm_y
		call	wm_p
		call	wm_a
		call	wm_d
		call	wm_k
		call	wm_u
		call	wm_space
		call	wm_s
		call	wm_i
		call	wm_e
		call	wm_t
		call	wm_e

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_v
		call	wm_1
		call	wm_point
		call	wm_1
		call	wm_0
		call	wm_space
		call	wm_minus
		call	wm_space
		call	wm_2
		call	wm_0
		call	wm_point
		call	wm_0
		call	wm_8
		call	wm_point
		call	wm_2
		call	wm_0
		call	wm_1
		call	wm_3

		return
;------------------------------------------------------------------------------------------
w_L2a		call	wm_8
		call	wm_9
;		goto	back1
;------------------------------------------------------------------------------------------
w_mode		movlw	B'11001111'		;row: 2 / col: 15
		call	send_comm
		return
;------------------------------------------------------------------------------------------
;
w_L2b		call	wm_L
		call	wm_2
		return
;------------------------------------------------------------------------------
w_wkbd		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_w
		call	wm_a
		call	wm_i
		call	wm_t
		call	wm_i
		call	wm_n
		call	wm_g
		call	wm_space
		call	wm_f
		call	wm_o
		call	wm_r
		call	wm_space
		call	wm_k
		call	wm_e
		call	wm_y
		call	wm_b
		call	wm_o
		call	wm_a
		call	wm_r
		call	wm_d
		call	wm_point
		call	wm_point
		call	wm_point

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
w_i2c_s		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_R
		call	wm_zat2
		call	wm_T
		call	wm_C
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_sls
		call	wm_space
		call	wm_zat1
		call	wm_T
		call	wm_zat2
		call	wm_e
		call	wm_m
		call	wm_p
		call	wm_e
		call	wm_r
		call	wm_a
		call	wm_t
		call	wm_u
		call	wm_r
		call	wm_e

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_E
		call	wm_zat2
		call	wm_E
		call	wm_P
		call	wm_R
		call	wm_O
		call	wm_M
		call	wm_space
		call	wm_sls
		call	wm_space
		call	wm_zat1
		call	wm_P
		call	wm_zat2
		call	wm_o
		call	wm_r
		call	wm_t
		call	wm_s

		return
;-------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------------------
w_rtcssc	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_1
		call	wm_zat2
		call	wm_D
		call	wm_S
		call	wm_1
		call	wm_3
		call	wm_0
		call	wm_7
		call	wm_sls
		call	wm_M
		call	wm_4
		call	wm_1
		call	wm_T
		call	wm_0
		call	wm_0
		call	wm_C
		call	wm_A
		call	wm_P

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_2
		call	wm_zat2
		call	wm_M
		call	wm_C
		call	wm_P
		call	wm_7
		call	wm_9
		call	wm_4
		call	wm_0
		call	wm_N

		return		
;---------------------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------------------
;RTC display date/time in first row
;
w_rtcm_dt	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		movf	v_rtc_d1,0
		call	send_data
		movf	v_rtc_d2,0
		call	send_data
		call	wm_bodka
		movf	v_rtc_m1,0
		call	send_data
		movf	v_rtc_m2,0
		call	send_data
		call	wm_bodka
		movf	v_rtc_y1,0
		call	send_data
		movf	v_rtc_y2,0
		call	send_data
		movf	v_rtc_y3,0
		call	send_data
		movf	v_rtc_y4,0
		call	send_data

		call	wm_sls
		
		movf	v_rtc_h1,0
		call	send_data
		movf	v_rtc_h2,0
		call	send_data
		call	wm_dvojbodka
		movf	v_rtc_n1,0
		call	send_data
		movf	v_rtc_n2,0
		call	send_data
		call	wm_dvojbodka
		movf	v_rtc_s1,0
		call	send_data
		movf	v_rtc_s2,0
		call	send_data

		call	wm_space

		movf	v_rtc_t1,0
		call	send_data
		movf	v_rtc_t2,0
		call	send_data

		return
;-----------------------------------------------------------------
w_rtcm		call	w_rtcm_dt
;
;
		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_S
		call	wm_zat2
		call	wm_e
		call	wm_t
		call	wm_sls
		call	wm_zat1
		call	wm_E
		call	wm_zat2
		call	wm_sls
		call	wm_E
		call	wm_S
		call	wm_C
		call	wm_space
		call	wm_space
		call	w_i2c_add

		return
;-----------------------------------------------------------------
w_rtcm_s	call	w_rtcm_dt

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_S
		call	wm_zat1
		call	wm_e
		call	wm_zat2
		call	wm_t
		call	wm_sls
		call	wm_E
		call	wm_S
		call	wm_C

		return
;-----------------------------------------------------------------
w_rtc_e		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_R
		call	wm_T
		call	wm_C
		call	wm_space
		call	wm_s
		call	wm_e
		call	wm_t
		call	wm_space
		call	wm_e
		call	wm_x
		call	wm_e
		call	wm_c
		call	wm_u
		call	wm_t
		call	wm_e
		call	wm_d
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka

		movlw	B'11000000'		;row: 1 / col: 0
		call	send_comm
		call	wm_P
		call	wm_r
		call	wm_e
		call	wm_s
		call	wm_s
		call	wm_space
		call	wm_E
		call	wm_N
		call	wm_T
		call	wm_E
		call	wm_R
		return
;-----------------------------------------------------------------
;-----------------------------------------------------------------
;-----------------------------------------------------------------
;-----------------------------------------------------------------
w_m001		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_zat1
		call	wm_I
		call	wm_zat2
		call	wm_2
		call	wm_C

		call	wm_space
		call	wm_sls
		call	wm_space
		call	wm_zat1
		call	wm_C
		call	wm_zat2
		call	wm_l
		call	wm_o
		call	wm_c
		call	wm_k

;nothing else implemented yet...

		return
;-----------------------------------------------------------------
;-----------------------------------------------------------------
w_gw1		movf	v_sdd1,0
		call	send_data
		movf	v_sdd2,0
		call	send_data
		call	wm_bodka
		movf	v_smm1,0
		call	send_data
		movf	v_smm2,0
		call	send_data
		movf	v_smm3,0
		call	send_data

		return
;-----------------------------------------------------------------
;-----------------------------------------------------------------
w_pl_nz		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_V
		call	wm_y
		call	wm_p
		call	wm_a
		call	wm_d
		call	wm_k
		call	wm_o
		call	wm_v
		call	wm_space
		call	wm_o
		call	wm_d
		call	wm_space
;
		call	w_gw1

		call	wm_dvojbodka
		call	wm_space
		movlw	'>'
		call	send_data

		movf	v_pl_cnt,0
		movwf	kbd_buffer
		call	write_dec

		movlw	'<'
		call	send_data

		return
;-----------------------------------------------------------------
;-----------------------------------------------------------------
w_pl_zero	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_Z
		call	wm_i
		call	wm_a
		call	wm_d
		call	wm_e
		call	wm_n
		call	wm_space
		call	wm_v
		call	wm_y
		call	wm_p
		call	wm_a
		call	wm_d
		call	wm_o
		call	wm_k
		call	wm_space
		call	wm_o
		call	wm_d
		call	wm_space
;
		call	w_gw1

		return
;-----------------------------------------------------------------------
;-----------------------------------------------------------------------
w_date_time	movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_hvizdicka
		call	wm_space

		movf	v_rtc_d1,0
		call	send_data
		movf	v_rtc_d2,0
		call	send_data
		call	wm_bodka
		movf	v_rtc_m1,0
		call	send_data
		movf	v_rtc_m2,0
		call	send_data
		call	wm_bodka
		movf	v_rtc_y1,0
		call	send_data
		movf	v_rtc_y2,0
		call	send_data
		movf	v_rtc_y3,0
		call	send_data
		movf	v_rtc_y4,0
		call	send_data

		call	wm_space
		call	wm_space
		
		movf	v_rtc_h1,0
		call	send_data
		movf	v_rtc_h2,0
		call	send_data
		call	wm_dvojbodka
		movf	v_rtc_n1,0
		call	send_data
		movf	v_rtc_n2,0
		call	send_data
		call	wm_dvojbodka
		movf	v_rtc_s1,0
		call	send_data
		movf	v_rtc_s2,0
		call	send_data

		call	wm_space
		call	wm_hvizdicka

		return
;-----------------------------------------------------------------------
;-----------------------------------------------------------------------
w_no_data	call	wm_z
		call	wm_i
		call	wm_a
		call	wm_d
		call	wm_n
		call	wm_e
		call	wm_space
		call	wm_u
		call	wm_d
		call	wm_a
		call	wm_j
		call	wm_e
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka
		return
;-----------------------------------------------------------------------
;-----------------------------------------------------------------------
w_pl_detail1	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_V
		call	wm_y
		call	wm_p
		call	wm_a
		call	wm_d
		call	wm_o
		call	wm_k
		call	wm_space

		movf	v_dnum,0
		movwf	kbd_buffer
		call	write_dec

		call	wm_dvojbodka
		call	wm_space

		movf	v_ts_day,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	w_no_data

		call	w_gw1
		call	wm_sls

		movf	v_rtc_h1,0
		call	send_data
		movf	v_rtc_h2,0
		call	send_data
		call	wm_dvojbodka
		movf	v_rtc_n1,0
		call	send_data
		movf	v_rtc_n2,0
		call	send_data

		return
;-----------------------------------------------------------------------
;-----------------------------------------------------------------------
w_pl_detail2	movlw	B'11001100'		;row: 2 / col: 12
		call	send_comm

		movf	v_ts_day,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	w_no_data


		call	w_gw1

		call	wm_sls

		movf	v_rtc_h1,0
		call	send_data
		movf	v_rtc_h2,0
		call	send_data
		call	wm_dvojbodka
		movf	v_rtc_n1,0
		call	send_data
		movf	v_rtc_n2,0
		call	send_data

		return
;-----------------------------------------------------------------------
;-----------------------------------------------------------------------
w_crs		movlw	B'10000000'
		call	send_comm

		call	wm_V
		call	wm_y
		call	wm_n
		call	wm_u
		call	wm_l
		call	wm_o
		call	wm_v
		call	wm_a
		call	wm_t
		call	wm_space
		call	wm_p
		call	wm_o
		call	wm_c
		call	wm_i
		call	wm_t
		call	wm_a
		call	wm_d
		call	wm_l
		call	wm_o
		call	wm_otaznik

		movlw	B'11000000'
		call	send_comm

		call	wm_a
		call	wm_l
		call	wm_e
		call	wm_b
		call	wm_o
		call	wm_space
		call	wm_p
		call	wm_r
		call	wm_e
		call	wm_p
		call	wm_n
		call	wm_u
		call	wm_t
		call	wm_space
		call	wm_n
		call	wm_a
		call	wm_s
		call	wm_p
		call	wm_a
		call	wm_t
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka

		return
;-----------------------------------------------------------------------
;-----------------------------------------------------------------------
w_crse		movlw	B'10000000'
		call	send_comm

		call	wm_S
		call	wm_t
		call	wm_a
		call	wm_v
		call	wm_space
		call	wm_v
		call	wm_y
		call	wm_n
		call	wm_u
		call	wm_l
		call	wm_o
		call	wm_v
		call	wm_a
		call	wm_n
		call	wm_y

		movlw	B'11000000'
		call	send_comm

		call	wm_P
		call	wm_r
		call	wm_e
		call	wm_p
		call	wm_n
		call	wm_u
		call	wm_t
		call	wm_space
		call	wm_s
		call	wm_p
		call	wm_a
		call	wm_t
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka

		return
;-----------------------------------------------------------------------
;-----------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

		END